import logging

from bitfield import BitField

from isc_common import setAttr, delAttr
from isc_common.bit import IsBitOn, TurnBitOn
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefManager, BaseRefQuerySet, BaseRef
from kaf_pas.sales.models.precent_item_types import Precent_item_types

logger = logging.getLogger(__name__)


class Precent_typesQuerySet(BaseRefQuerySet):
    pass


class Precent_typesManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'precent_item_type_id': record.precent_item_type.id,
            'precent_item_type__code': record.precent_item_type.code,
            'precent_item_type__name': record.precent_item_type.name,
            'description': record.description,
            'props': record.props._value,
            'enableDogovor': IsBitOn(record.props, 0),
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Precent_typesQuerySet(self.model, using=self._db)

    def createFromRequest(self, request):
        _request = DSRequest(request=request)
        data = _request.get_data()
        _data = data.copy()

        props = 0
        if _data.get('enableDogovor'):
            props = TurnBitOn(props, 0)

        delAttr(_data, 'enableDogovor')
        delAttr(_data, 'precent_item_type__name')
        setAttr(_data, 'props', props)
        res = super().create(**_data)
        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        id = data.get('id')

        props = 0
        if data.get('enableDogovor'):
            props = TurnBitOn(props, 0)

        delAttr(data, 'id')
        delAttr(data, 'enableDogovor')
        delAttr(data, 'precent_item_type__code')
        delAttr(data, 'precent_item_type__name')
        setAttr(data, 'props', props)
        res = super().filter(id=id).update(**data)
        return res


class Precent_types(BaseRef):
    precent_item_type = ForeignKeyProtect(Precent_item_types)
    props = BitField(flags=(
        ('enableDogovor', 'Наличие договора'),
    ), default=1, db_index=True)

    objects = Precent_typesManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Типы распоряжений'



# Generated by Django 3.0.3 on 2020-02-17 04:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0087_auto_20200209_1244'),
    ]

    operations = [
        migrations.CreateModel(
            name='Routing',
            fields=[
            ],
            options={
                'verbose_name': 'Маршрутизация',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('planing.operations_view',),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-28 15:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0061_auto_20190525_0953'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='item_line_view',
            table='ckk_item_line_mview',
        ),
        migrations.AlterModelTable(
            name='item_view',
            table='ckk_item_mveiw',
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-17 08:13

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0183_auto_20200117_0811'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_refs',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('relevant_in_planing', 'Учавствует в планировании')), db_index=True, default=3),
        ),
    ]



import logging

from django.db.models import BooleanField

from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import BaseRefHierarcy
from isc_common.number import DelProps
from kaf_pas.production.models.operations import OperationsManager

logger = logging.getLogger(__name__)


class Operations_viewQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operations_viewManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "code": record.code,
            "name": record.name,
            "full_name": record.full_name,
            "description": record.description,
            "parent_id": record.parent_id,
            "lastmodified": record.lastmodified,
            'isFolder': record.isFolder,
            'editing': record.editing,
            'deliting': record.deliting,
            'props': record.props,
            'assemly': record.props.assemly,
            'made_common_form': record.props.made_common_form,
        }
        return DelProps(res)

    def get_queryset(self):
        return Operations_viewQuerySet(self.model, using=self._db)


class Operations_view(BaseRefHierarcy):
    isFolder = BooleanField()
    props = OperationsManager.get_props()

    objects = Operations_viewManager()

    def _get_planing_operation_type(self, parent):
        if parent:
            if parent.planing_operation_type:
                return parent.planing_operation_type
            else:
                return self._get_planing_operation_type(parent=parent.parent)
        else:
            return None

    def __str__(self):
        return f"ID={self.id}, code={self.code}, name={self.name}, description={self.description}"

    class Meta:
        db_table = 'production_operations_view'
        managed = False
        verbose_name = 'Операции'



# Generated by Django 2.2.1 on 2019-05-23 11:25

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0054_auto_20190521_2304'),
    ]

    operations = [
        migrations.AlterField(
            model_name='status',
            name='code_owner',
            field=isc_common.fields.code_field.CodeField(),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-20 21:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0005_auto_20200119_1744'),
    ]

    operations = [
        migrations.AddField(
            model_name='tmp_buffer',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attributes_STMP_1

urlpatterns = [

    path('Document_attributes_STMP_1/Fetch/', document_attributes_STMP_1.Document_attributes_STMP_1_Fetch),
    path('Document_attributes_STMP_1/Add', document_attributes_STMP_1.Document_attributes_STMP_1_Add),
    path('Document_attributes_STMP_1/Update', document_attributes_STMP_1.Document_attributes_STMP_1_Update),
    path('Document_attributes_STMP_1/Remove', document_attributes_STMP_1.Document_attributes_STMP_1_Remove),
    path('Document_attributes_STMP_1/Lookup/', document_attributes_STMP_1.Document_attributes_STMP_1_Lookup),
    path('Document_attributes_STMP_1/Info/', document_attributes_STMP_1.Document_attributes_STMP_1_Info),
    path('Document_attributes_STMP_1/Copy', document_attributes_STMP_1.Document_attributes_STMP_1_Copy),

]



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_COUNT

urlpatterns = [

    path('Document_attributes_SPC_CLM_COUNT/Fetch/', document_attributes_SPC_CLM_COUNT.Document_attributes_SPC_CLM_COUNT_Fetch),
    path('Document_attributes_SPC_CLM_COUNT/Add', document_attributes_SPC_CLM_COUNT.Document_attributes_SPC_CLM_COUNT_Add),
    path('Document_attributes_SPC_CLM_COUNT/Update', document_attributes_SPC_CLM_COUNT.Document_attributes_SPC_CLM_COUNT_Update),
    path('Document_attributes_SPC_CLM_COUNT/Remove', document_attributes_SPC_CLM_COUNT.Document_attributes_SPC_CLM_COUNT_Remove),
    path('Document_attributes_SPC_CLM_COUNT/Lookup/', document_attributes_SPC_CLM_COUNT.Document_attributes_SPC_CLM_COUNT_Lookup),
    path('Document_attributes_SPC_CLM_COUNT/Info/', document_attributes_SPC_CLM_COUNT.Document_attributes_SPC_CLM_COUNT_Info),
    path('Document_attributes_SPC_CLM_COUNT/Copy', document_attributes_SPC_CLM_COUNT.Document_attributes_SPC_CLM_COUNT_Copy),

]



import requests
from django.conf import settings
from django.http import HttpResponseRedirect

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy, Lotsman_documents_hierarcyManager
from kaf_pas.kd.models.lotsman_documents_hierarcy_ext import Lotsman_documents_hierarcyManagerExt
from kaf_pas.kd.models.lotsman_documents_hierarcy_view import Lotsman_documents_hierarcy_view, Lotsman_documents_hierarcy_viewManager


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Lotsman_documents_hierarcy_view.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Lotsman_documents_hierarcy_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Fetch1(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Lotsman_documents_hierarcy_view.objects.
                distinct('id').
                get_range_rows1(
                request=request,
                function=Lotsman_documents_hierarcy_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_doc_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Lotsman_documents_hierarcy_view.objects.
                filter().
                get_range_rows4(
                request=request,
                function=Lotsman_documents_hierarcy_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Add(request):
    return JsonResponse(DSResponseAdd(data=Lotsman_documents_hierarcy.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Update(request):
    return JsonResponse(DSResponseUpdate(data=Lotsman_documents_hierarcy.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_documents_hierarcy.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_documents_hierarcy.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Info(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_documents_hierarcy.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_documents_hierarcy.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_MakeItem(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_documents_hierarcy.objects.makeItemFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcy_ReloadDoc(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    ids = _request.json.get('data')
    ids = ids.get('ids')

    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Lotsman_documents_hierarcy/ReloadDoc', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host, user_id=_request.user_id))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_documents_hierarcys_RefreshMView(request):
    _request = DSRequest(request=request)
    Lotsman_documents_hierarcyManagerExt(user=_request.user).make_mview()
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.8 on 2019-12-06 13:29

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0027_auto_20191206_1329'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_types_users',
            name='operation_type',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='planing.Operation_types'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-07 17:21

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0078_auto_20190606_1839'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item_line',
            name='isFolder',
        ),
        migrations.AlterField(
            model_name='item',
            name='STMP_1',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='STMP_1', to='kd.Document_attributes', verbose_name='Наименование изделия'),
        ),
        migrations.AlterField(
            model_name='item',
            name='STMP_2',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='STMP_2', to='kd.Document_attributes', verbose_name='Обозначение изделия'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_COUNT',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_COUNT', to='kd.Document_attributes', verbose_name='Количество'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_FACTORY',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_FACTORY', to='kd.Document_attributes', verbose_name='Предприятие - изготовитель'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_FORMAT',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_FORMAT', to='kd.Document_attributes', verbose_name='Формат'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_KOD',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_KOD', to='kd.Document_attributes', verbose_name='Код'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_MARK',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MARK', to='kd.Document_attributes', verbose_name='Обозначение'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_MASSA',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MASSA', to='kd.Document_attributes', verbose_name='Масса'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_MATERIAL',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MATERIAL', to='kd.Document_attributes', verbose_name='Материал'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_NAME',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_NAME', to='kd.Document_attributes', verbose_name='Наименование'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_NOTE',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_NOTE', to='kd.Document_attributes', verbose_name='Примечание'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_POS',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_POS', to='kd.Document_attributes', verbose_name='Позиция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_USER',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_USER', to='kd.Document_attributes', verbose_name='Пользовательская'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='SPC_CLM_ZONE',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_ZONE', to='kd.Document_attributes', verbose_name='Зона'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='section',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='section', to='kd.Document_attributes', verbose_name='Секция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='subsection',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='subsection', to='kd.Document_attributes', verbose_name='Подсекция'),
        ),
    ]



from django.db.models import Max

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.material_type import Material_type, Material_typeManager
from kaf_pas.ckk.models.material_type_view import Material_type_view, Material_type_viewManager


@JsonResponseWithException()
def Material_type_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Material_type_view.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Material_type_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_type_Add(request):
    return JsonResponse(DSResponseAdd(data=Material_type.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_type_Update(request):
    return JsonResponse(DSResponseUpdate(data=Material_type.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_type_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Material_type.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_type_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Material_type.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_type_Info(request):
    return JsonResponse(DSResponse(request=request, data=Material_type.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_type_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Material_type.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_type_CopyParams(request):
    _request = DSRequest(request=request)
    source = _request.json.get('source')
    destination = _request.json.get('destination')
    res = False

    if isinstance(source, dict) and isinstance(destination, dict):
        srecord = source.get('record')
        drecord = destination.get('record')

        if srecord and drecord:
            from kaf_pas.ckk.models.material_params import Material_params
            order = Material_params.objects.filter(materials_type=drecord.get(id))
            max_order = order.aggregate(Max('order'))
            max_order = max_order.get('max_order', 0)
            for params in Material_params.objects.filter(materials_type_id=srecord.get('id')):
                max_order += 1
                Material_params.objects.get_or_create(materials_type_id=drecord.get('id'), param_type=params.param_type, order=max_order)
                res = True

    if res:
        return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)
    else:
        raise Exception(f'Копирование не выполнено.')



# Generated by Django 2.2.7 on 2019-11-16 13:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0014_auto_20191013_1142'),
    ]

    operations = [
        migrations.AddField(
            model_name='user_add_info',
            name='ip_address',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
        migrations.AddField(
            model_name='user_add_info',
            name='user_agent',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-11 11:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0092_auto_20190611_1151'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cdw_pdf_item_view',
            options={'managed': False, 'verbose_name': 'Чертеж'},
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import item_line

urlpatterns = [

    path('Item_line/Fetch/', item_line.Item_line_Fetch),
    path('Item_line/Add', item_line.Item_line_Add),
    path('Item_line/Update', item_line.Item_line_Update),
    path('Item_line/Copy', item_line.Item_line_Copy),
    path('Item_line/Remove', item_line.Item_line_Remove),
    path('Item_line/Lookup/', item_line.Item_line_Lookup),
    path('Item_line/Info/', item_line.Item_line_Info),
    # path('Item_line/Copy', item_line.Item_line_Copy),

]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0146_material_askon_mview'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='attr_type',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='ed_izm',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='files_askon',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='item',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='item_image_refs',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='item_line',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='item_location',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='item_refs',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='locations',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='material_added_values',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='material_askon',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='material_params',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='material_type',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='materials',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='status',
            name='id_old',
        ),
    ]



from django.urls import path

from kaf_pas.input.views import users_ex

urlpatterns = [

    path('Users_ex/Fetch/', users_ex.Users_ex_Fetch),
    path('Users_ex/Add', users_ex.Users_ex_Add),
    path('Users_ex/Update', users_ex.Users_ex_Update),
    path('Users_ex/Remove', users_ex.Users_ex_Remove),
    path('Users_ex/Lookup/', users_ex.Users_ex_Lookup),
    path('Users_ex/Info/', users_ex.Users_ex_Info),
    path('Users_ex/Copy', users_ex.Users_ex_Copy),

]



# Generated by Django 2.2.6 on 2019-10-20 18:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0157_item_line_section_num'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Status',
        ),
    ]



import logging

from django.db import connection, transaction, ProgrammingError
from isc_common import setAttr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from refs.models.type_param import Type_param
from refs.models.type_param_values import Type_param_values

from kaf_pas.ckk.models.material_params import Material_params
from kaf_pas.ckk.models.materials import Materials

logger = logging.getLogger(__name__)


class Material_added_valuesQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Material_added_valuesManager(AuditManager):

    def get_range_rows1(self, request, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        fields = []
        material = Materials.objects.get(id=data.get('id'))
        view_name = f'ckk_{material.materials_type.code}_view'

        for material_params in Material_params.objects.filter(materials_type_id=data.get('materials_type_id')).order_by('order', ):
            fields.append(material_params.param_type.code)

        sql_str = f'''select {",".join(fields)} from {view_name} where id = {data.get("id")}'''
        result = []
        try:
            with connection.cursor() as cursor:
                cursor.execute(sql_str)

                i = 0
                _row = dict()
                for row in cursor.fetchall():
                    for field in fields:
                        setAttr(_row, field, row[i])
                        i += 1
                result.append(_row)
        except ProgrammingError:
            ...

        return result

    def get_queryset(self):
        return Material_added_valuesQuerySet(self.model, using=self._db)

    def updateFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        data = request.get_data()

        material_id = data.get('id')
        materials_type_id = data.get('materials_type_id')

        with transaction.atomic():
            for material_params in Material_params.objects.filter(materials_type_id=materials_type_id).order_by('order', ):
                value = data.get(material_params.param_type.code)

                try:
                    material_added_value = self.get(material_id=material_id, param_values__type__code=material_params.param_type.code)
                    if material_params.param_type.type == Type_param.integer():
                        material_added_value.param_values.value_int = value
                        material_added_value.param_values.save()
                    elif material_params.param_type.type == Type_param.float():
                        material_added_value.param_values.value_float = value
                        material_added_value.param_values.save()
                    elif material_params.param_type.type == Type_param.date():
                        material_added_value.param_values.value_date = value
                        material_added_value.param_values.save()
                    elif material_params.param_type.type == Type_param.date_time():
                        material_added_value.param_values.value_date = value
                        material_added_value.param_values.save()
                    elif material_params.param_type.type == Type_param.string():
                        material_added_value.param_values.value_str = value
                        material_added_value.param_values.save()
                    elif material_params.param_type.type == Type_param.select():
                        material_added_value.param_values.value_str = value
                        material_added_value.param_values.save()
                    else:
                        raise Exception(f'{material_params.param_type.type}: unknown type.')

                except Material_added_values.DoesNotExist:
                    if material_params.param_type.type == Type_param.integer():
                        self.create(material_id=material_id, param_values=Type_param_values.objects.create(type=material_params.param_type, value_int=value))
                    elif material_params.param_type.type == Type_param.float():
                        self.create(material_id=material_id, param_values=Type_param_values.objects.create(type=material_params.param_type, value_float=value))
                    elif material_params.param_type.type == Type_param.date():
                        self.create(material_id=material_id, param_values=Type_param_values.objects.create(type=material_params.param_type, value_date=value))
                    elif material_params.param_type.type == Type_param.date_time():
                        self.create(material_id=material_id, param_values=Type_param_values.objects.create(type=material_params.param_type, value_date=value))
                    elif material_params.param_type.type == Type_param.string():
                        self.create(material_id=material_id, param_values=Type_param_values.objects.create(type=material_params.param_type, value_str=value))
                    elif material_params.param_type.type == Type_param.select():
                        self.create(material_id=material_id, param_values=Type_param_values.objects.create(type=material_params.param_type, value_str=value))
                    else:
                        raise Exception(f'{material_params.param_type.type}: unknown type.')
        return data


class Material_added_values(AuditModel):
    material = ForeignKeyProtect(Materials)
    param_values = ForeignKeyProtect(Type_param_values)

    objects = Material_added_valuesManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Значения дополнительных (настраиваимых) параметров'



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.demand_materials import Demand_materials, Demand_materialsManager


@JsonResponseWithException()
def Demand_materials_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Demand_materials.objects.
                select_related('demand', 'precent_material').
                get_range_rows1(
                request=request,
                function=Demand_materialsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_materials_Add(request):
    return JsonResponse(DSResponseAdd(data=Demand_materials.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_materials_Update(request):
    return JsonResponse(DSResponseUpdate(data=Demand_materials.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_materials_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Demand_materials.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_materials_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Demand_materials.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_materials_Info(request):
    return JsonResponse(DSResponse(request=request, data=Demand_materials.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_materials_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Demand_materials.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from django.urls import path

from kaf_pas.kd.views import pathes_view

urlpatterns = [

    path('Pathes_view/Fetch/', pathes_view.Pathes_view_Fetch),
    path('Pathes_view/Add', pathes_view.Pathes_view_Add),
    path('Pathes_view/Update', pathes_view.Pathes_view_Update),
    path('Pathes_view/Remove', pathes_view.Pathes_view_Remove),
    path('Pathes_view/Lookup/', pathes_view.Pathes_view_Lookup),
    path('Pathes_view/Info/', pathes_view.Pathes_view_Info),

]



import logging

from isc_common.fields.related import ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsQuerySet, CommonManagetWithLookUpFieldsManager
from isc_common.models.audit import AuditModel
from kaf_pas.sales.models.dogovors import Dogovors
from kaf_pas.sales.models.precent import Precent

logger = logging.getLogger(__name__)


class Precent_dogovorsQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    pass


class Precent_dogovorsManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'dogovor_id': record.dogovor.id,
            'dogovor__name': record.dogovor.name,
            'dogovor__full_name': record.dogovor.full_name,
            'dogovor__date': record.dogovor.date,
            'editing': record.editing,
            'deliting': record.deliting,
            # 'enabled': IsBitOn(record.dogovor.props, 0)
        }
        return res

    def get_queryset(self):
        return Precent_dogovorsQuerySet(self.model, using=self._db)


class Precent_dogovors(AuditModel):
    dogovor = ForeignKeyCascade(Dogovors)
    precent = ForeignKeyCascade(Precent)

    objects = Precent_dogovorsManager()

    def __str__(self):
        return f"ID:{self.id}, dogovor: [{self.dogovor}], precent: {self.precent}"

    class Meta:
        verbose_name = 'Кросс-таблица'



from django.urls import path

from kaf_pas.accounting.views import buffers

urlpatterns = [

    path('Buffers/Fetch/', buffers.Buffers_Fetch),
    path('Buffers/Add', buffers.Buffers_Add),
    path('Buffers/Update', buffers.Buffers_Update),
    path('Buffers/Remove', buffers.Buffers_Remove),
    path('Buffers/Lookup/', buffers.Buffers_Lookup),
    path('Buffers/Info/', buffers.Buffers_Info),
    path('Buffers/Copy', buffers.Buffers_Copy),

]



from django.urls import path

from kaf_pas.ckk.views import item_view

urlpatterns = [

    path('ItemView/Fetch/', item_view.Item_view_Fetch),
    path('ItemView/Fetch1/', item_view.Item_view_Fetch1),
    path('ItemView/Fetch2/', item_view.Item_view_Fetch2),
    path('ItemView/Fetch3/', item_view.Item_view_Fetch3),
    path('ItemView/Fetch4/', item_view.Item_view_Fetch4),
    path('ItemView/Add', item_view.Item_view_Add),
    path('ItemView/Update', item_view.Item_view_Update),
    path('ItemView/Update1', item_view.Item_view_Update1),
    path('ItemView/Remove', item_view.Item_view_Remove),
    path('ItemView/Lookup/', item_view.Item_view_Lookup),
    path('ItemView/Info/', item_view.Item_view_Info),
    path('ItemView/Copy', item_view.Item_view_Copy),
    path('ItemView/CopyItems', item_view.copyItems),
    path('ItemView/CombineItems', item_view.combineItems),
    path('ItemView/CopyBlockItems', item_view.Item_view_CopyBlockItems),

]



# Generated by Django 2.2.8 on 2019-12-02 12:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0134_auto_20191121_2223'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='launch_item_refs',
            options={'verbose_name': 'Древо производственной спецификации'},
        ),
    ]



import logging

from django.forms import model_to_dict

from isc_common.http.DSRequest import DSRequest
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.planing.models.operations_view import Operations_view, Operations_viewManager, Operations_viewQuerySet

logger = logging.getLogger(__name__)


class RoutingQuerySet(Operations_viewQuerySet):
    def raw(self, raw_query=None, params=None, translations=None, using=None, function=None):
        if raw_query == None:
            raw_query = '''select *
                                    from (with r as (select distinct pr."id",
                                                         pr."child_id",
                                                         null::bigint "parent_id",
                                                         pr."num",
                                                         pr."date",
                                                         pr."deleted_at",
                                                         pr."editing",
                                                         pr."deliting",
                                                         pr."lastmodified",
                                                         pr."opertype_id",
                                                         pr."status_id",
                                                         pr."prev_status_id",
                                                         pr."description",
                                                         pr."value_id",
                                                         pr."item_id",
                                                         pr."item_full_name",
                                                         pr."launch_id",
                                                         pr."color_id",
                                                         pr."creator_id",
                                                         pr."operation_level_id",
                                                         pr."operation_level_code",
                                                         pr."operation_level_name",
                                                         pr."resource_id",
                                                         pr."resource_location_id",
                                                         pr."resource_name",
                                                         0 as "props",
                                                         pr."production_operation_id",
                                                         pr."production_operation_num",
                                                         pr."production_operation_qty",
                                                         pr."production_operation_edizm_id",
                                                         pr."mark",
                                                         pr."isFolder"
                                         from planing_operations_view pr
                                                  join planing_operation_resources por on pr.id = por.operation_id
                                                  join production_resource pr1 on por.resource_id = pr1.id
                                         where pr.item_id = %s
                                           and por.resource_id = %s
                                           and pr.props = 1
                                           )
                                    
                                          select distinct poi."id",
                                               pr."child_id",
                                               null::bigint as "parent_id",
                                               poi."num",
                                               poi."date",
                                               poi."deleted_at",
                                               poi."editing",
                                               poi."deliting",
                                               poi."lastmodified",
                                               poi."opertype_id",
                                               poi."status_id",
                                               poi."prev_status_id",
                                               poi."description",
                                               poi."value_id",
                                               poi."item_id",
                                               poi."item_full_name",
                                               poi."launch_id",
                                               poi."color_id",
                                               poi."creator_id",
                                               poi."operation_level_id",
                                               poi."operation_level_code",
                                               poi."operation_level_name",
                                               poi."resource_id",
                                               poi."resource_location_id",
                                               poi."resource_name",
                                               poi."props",
                                               poi."production_operation_id",
                                               poi."production_operation_num",
                                               poi."production_operation_qty",
                                               poi."production_operation_edizm_id",
                                               poi."isFolder",
                                               'income'       as mark
                                        from planing_operations_view pr
                                        left join planing_operations_view poi on pr.parent_id = poi.id
                                        where pr.child_id = (select distinct poi.id
                                                          from planing_operations_view poi
                                                          where poi.item_id = %s
                                                            and poi.opertype_id = %s
                                                            and poi.production_operation_num = (select min(production_operation_num)
                                                                           from planing_operations_view poi
                                                                           where poi.item_id = %s
                                                                             and poi.resource_id = %s
                                                                             and poi.opertype_id = %s
                                                          ))
                                          and pr.props = 2
                                          union
                                          select pr."id",
                                                 pr."child_id",
                                                 pr."parent_id",
                                                 pr."num",
                                                 pr."date",
                                                 pr."deleted_at",
                                                 pr."editing",
                                                 pr."deliting",
                                                 pr."lastmodified",
                                                 pr."opertype_id",
                                                 pr."status_id",
                                                 pr."prev_status_id",
                                                 pr."description",
                                                 pr."value_id",
                                                 pr."item_id",
                                                 pr."item_full_name",
                                                 pr."launch_id",
                                                 pr."color_id",
                                                 pr."creator_id",
                                                 pr."operation_level_id",
                                                 pr."operation_level_code",
                                                 pr."operation_level_name",
                                                 pr."resource_id",
                                                 pr."resource_location_id",
                                                 pr."resource_name",
                                                 pr."props",
                                                 pr."production_operation_id",
                                                 pr."production_operation_num",
                                                 pr."production_operation_qty",
                                                 pr."production_operation_edizm_id",
                                                 pr."isFolder",
                                                 'local' as mark
                                          from r pr
                                          union
                                          select pr."id",
                                                 pr."child_id",
                                                 null  as "parent_id",
                                                 pr."num",
                                                 pr."date",
                                                 pr."deleted_at",
                                                 pr."editing",
                                                 pr."deliting",
                                                 pr."lastmodified",
                                                 pr."opertype_id",
                                                 pr."status_id",
                                                 pr."prev_status_id",
                                                 pr."description",
                                                 pr."value_id",
                                                 pr."item_id",
                                                 pr."item_full_name",
                                                 pr."launch_id",
                                                 pr."color_id",
                                                 pr."creator_id",
                                                 pr."operation_level_id",
                                                 pr."operation_level_code",
                                                 pr."operation_level_name",
                                                 pr."resource_id",
                                                 pr."resource_location_id",
                                                 pr."resource_name",
                                                 pr."props",
                                                 pr."production_operation_id",
                                                 pr."production_operation_num",
                                                 pr."production_operation_qty",
                                                 pr."production_operation_edizm_id",
                                                 pr."isFolder",
                                                 'outcome'      as mark
                                          from planing_operations_view pr
                                          where parent_id = (select distinct poo.operation_id
                                                             from planing_operation_item poi
                                                                      join planing_operation_operation poo on poi.operation_id = poo.operation_id
                                                                      join planing_operations_view op on poi.operation_id = op.id
                                                             where poi.item_id = %s
                                                               and op.opertype_id = %s
                                                               and poo.num = (select max(production_operation_num)
                                                                              from planing_operations_view poi
                                                                              where poi.item_id = %s
                                                                                and poi.resource_id = %s
                                                                                and poi.opertype_id = %s
                                                             ))
                                            and pr.props = 2) as a
                                    '''

        queryResult = super().raw(raw_query=raw_query, params=params, translations=translations, using=using)
        if function:
            res = [function(record) for record in queryResult]
        else:
            res = [model_to_dict(record) for record in queryResult]
        return res


class RoutingManager(Operations_viewManager):

    @staticmethod
    def getRecord(record):
        return Operations_viewManager.getRecord(record)

    def get_queryset(self):
        return RoutingQuerySet(self.model, using=self._db)

    def fetchLevelsFromRequest(self, request):
        request = DSRequest(request=request)

        launch_id = request.get_data().get('launch_id')
        levels = RoutingManager.make_levels(launch_id=launch_id)
        return levels

    @staticmethod
    def make_levels(launch_id):
        res = [
            dict(
                id=operation.get('operation_level_id'),
                title=operation.get('operation_level__name'),
                prompt=f'''ID: {operation.get('operation_level_id')}, {operation.get('operation_level__code')} : {operation.get('operation_level__name')}'''
            )
            for operation in Operations_view.objects.
                filter(
                launch_id=launch_id,
                props__in=[
                    Operations_view.props.inner_routing,
                    Operations_view.props.outer_routing,
                ]
            ).
                order_by('operation_level__code').
                values('operation_level_id', 'operation_level__name', 'operation_level__code').
                distinct()
        ]
        return res

    def fetchLocationsLevelFromRequest(self, request):
        request = DSRequest(request=request)

        launch_id = request.get_data().get('launch_id')
        level_id = request.get_data().get('level_id')
        levels = RoutingManager.make_locationsLevel(launch_id=launch_id, level_id=level_id)
        return levels

    @staticmethod
    def make_locationsLevel(launch_id, level_id):
        res = sorted([
            dict(
                id=operation.get('resource__location_id'),
                title=Locations.objects.get(id=operation.get('resource__location_id')).full_name,
                # prompt=Locations.objects.get(id=operation.get('resource__location_id')).full_name,
                prompt=f'''ID: {operation.get('resource__location_id')}''',
            )
            for operation in Operations_view.objects.
                filter(
                launch_id=launch_id,
                props__in=[
                    Operations_view.props.inner_routing,
                    Operations_view.props.outer_routing,
                ],
                operation_level_id=level_id
            ).
                values('resource__location_id', 'resource__location__name').
                distinct()
        ],
            key=lambda x: x['title'])

        return res

    def fetchResourcesLevelFromRequest(self, request):
        request = DSRequest(request=request)

        launch_id = request.get_data().get('launch_id')
        level_id = request.get_data().get('level_id')
        location_id = request.get_data().get('location_id')
        levels = RoutingManager.make_resourcesLevel(launch_id=launch_id, level_id=level_id, location_id=location_id)
        return levels

    @staticmethod
    def make_resourcesLevel(launch_id, level_id, location_id):
        res = sorted([
            dict(
                id=operation.get('resource_id'),
                title=operation.get('resource__name'),
                prompt=f'''ID: {operation.get('resource_id')}, {operation.get('resource__description')}''',
            )
            for operation in Operations_view.objects.
                filter(
                launch_id=launch_id,
                operation_level_id=level_id,
                resource__location_id=location_id,
                props__in=[
                    Operations_view.props.inner_routing,
                    Operations_view.props.outer_routing,
                ],
            ).
                values('resource_id', 'resource__name', 'resource__description').
                distinct()
        ],
            key=lambda x: x['title'])

        return res


class Routing(Operations_view):
    objects = RoutingManager()

    def __str__(self):
        return super().__str__()

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Маршрутизация'
        proxy = True



# Generated by Django 2.2.6 on 2019-10-29 14:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0106_auto_20191029_1413'),
    ]

    operations = [
        migrations.RenameField(
            model_name='dogovors',
            old_name='code',
            new_name='name',
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-18 21:36

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0026_auto_20190518_1703'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document_attrs',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'),), db_index=True, default=1),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException, JsonWSResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.dogovor_download_file import download_dogovor_file
from kaf_pas.sales.models.dogovor_upload_file import DSResponse__Dogovor_UploadFile
from kaf_pas.sales.models.dogovors import Dogovors
from kaf_pas.sales.models.dogovors_view import Dogovors_view, Dogovors_viewManager


@JsonResponseWithException()
def Dogovors_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Dogovors_view.objects.
                select_related('status', 'customer').
                filter(props=Dogovors_view.props.real).
                get_range_rows1(
                request=request,
                function=Dogovors_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovors_Add(request):
    return JsonResponse(DSResponseAdd(data=Dogovors.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovors_Update(request):
    return JsonResponse(DSResponseUpdate(data=Dogovors.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovors_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Dogovors.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovors_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Dogovors.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovors_Info(request):
    return JsonResponse(DSResponse(request=request, data=Dogovors.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovors_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Dogovors.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonWSResponseWithException()
def Dogovor_UploadFile(request):
    return JsonResponse(DSResponse__Dogovor_UploadFile(request).response)


@JsonWSResponseWithException(printing=False)
def Dogovort_DownloadFile(request):
    return download_dogovor_file(request)



# Generated by Django 3.0.3 on 2020-02-22 05:48

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0210_auto_20200212_0752'),
        ('planing', '0104_auto_20200222_0527'),
    ]

    operations = [
        # migrations.AddField(
        #     model_name='operations_view',
        #     name='std_material',
        #     field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Materials'),
        # ),
    ]



# Generated by Django 2.2.1 on 2019-05-22 07:46

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0016_auto_20190521_2028'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='precent',
            options={'verbose_name': 'Документы'},
        ),
        migrations.AddField(
            model_name='precent',
            name='type',
            field=isc_common.fields.code_field.CodeField(),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-13 13:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0094_auto_20191213_1323'),
    ]

    operations = [
    ]



from django.urls import path

from kaf_pas.ckk.views import item_qty

urlpatterns = [

    path('Item_qty/Fetch/', item_qty.Item_qty_Fetch),
    path('Item_qty/Add', item_qty.Item_qty_Add),
    path('Item_qty/Update', item_qty.Item_qty_Update),
    path('Item_qty/Remove', item_qty.Item_qty_Remove),
    path('Item_qty/Lookup/', item_qty.Item_qty_Lookup),
    path('Item_qty/Info/', item_qty.Item_qty_Info),
    path('Item_qty/Copy', item_qty.Item_qty_Copy),

]



# Generated by Django 2.2.8 on 2019-12-15 01:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0108_auto_20191215_0105'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lotsman_documents_hierarcy',
            name='id',
            field=models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор'),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.item import Item, ItemManager


@JsonResponseWithException()
def Item_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Item.objects.
                filter().
                get_range_rows1(
                request=request,
                function=ItemManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_Add(request):
    return JsonResponse(DSResponseAdd(data=Item.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_Update(request):
    return JsonResponse(DSResponseUpdate(data=Item.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_Info(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_CheckRecursives(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.checkRecursives(request=request), status=RPCResponseConstant.statusSuccess).response)\

@JsonResponseWithException()
def GetQtyChilds(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.getQtyChilds(request=request), status=RPCResponseConstant.statusSuccess).response)





# Generated by Django 2.2.8 on 2019-12-04 20:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0146_auto_20191204_2015'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='launch_operations_item',
            name='q_paralel',
        ),
        migrations.RemoveField(
            model_name='operations_item',
            name='q_paralel',
        ),
        migrations.AddField(
            model_name='operation_resources',
            name='q_paralel',
            field=models.PositiveIntegerField(default=1),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-22 19:23

from django.db import migrations
import kaf_pas.input.models.users_ex


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0034_standard_colors_color'),
        ('input', '0016_user_agg_view'),
    ]

    operations = [
        migrations.CreateModel(
            name='Users_ex',
            fields=[
            ],
            options={
                'verbose_name': 'Пользователи с должностями',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('isc_common.user',),
            managers=[
                ('objects', kaf_pas.input.models.users_ex.Users_exManager()),
            ],
        ),
    ]



import logging

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.planing.models.operations import Operations

logger = logging.getLogger(__name__)


class Operation_executorQuerySet(AuditQuerySet):
    pass


class Operation_executorManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
            'executor_id': record.executor.id,
        }
        return res

    def get_queryset(self):
        return Operation_executorQuerySet(self.model, using=self._db)


class Operation_executor(AuditModel):
    operation = ForeignKeyCascade(Operations)
    executor = ForeignKeyProtect(User)

    objects = Operation_executorManager()

    def __str__(self):
        return f"ID:{self.id}, operation: [{self.operation}], executor: [{self.executor}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица значений операции'
        unique_together = (('operation', 'executor'),)



# Generated by Django 2.2.1 on 2019-05-21 15:17

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0013_auto_20190517_1641'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='precent',
            name='name',
        ),
        migrations.AddField(
            model_name='precent',
            name='customer',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='sales.Customer'),
        ),
        migrations.AddField(
            model_name='precent',
            name='date',
            field=models.DateTimeField(default=None, verbose_name='Дата'),
        ),
        migrations.AddField(
            model_name='precent',
            name='date_sign',
            field=models.DateTimeField(default=None, verbose_name='Дата подписания'),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-22 06:06

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0007_tmp_buffer_value_off'),
    ]

    operations = [
        migrations.AddField(
            model_name='tmp_buffer',
            name='color',
            field=isc_common.fields.code_field.ColorField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 00:56

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0042_auto_20190610_0047'),
    ]

    operations = [
        migrations.AddField(
            model_name='document_attr_cross',
            name='section',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Раздел'),
        ),
        migrations.AddField(
            model_name='document_attr_cross',
            name='subsection',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Подраздел'),
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operation_types

urlpatterns = [

    path('Operation_types/Fetch/', operation_types.Operation_types_Fetch),
    path('Operation_types/Add', operation_types.Operation_types_Add),
    path('Operation_types/Update', operation_types.Operation_types_Update),
    path('Operation_types/Remove', operation_types.Operation_types_Remove),
    path('Operation_types/Lookup/', operation_types.Operation_types_Lookup),
    path('Operation_types/Info/', operation_types.Operation_types_Info),
    path('Operation_types/Copy', operation_types.Operation_types_Copy),

]



import logging

from django.db.models import UniqueConstraint, Q
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditManager, AuditQuerySet, AuditModel

from kaf_pas.ckk.models.item import Item
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Item_documentQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_documentManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Item_documentQuerySet(self.model, using=self._db)


class Item_document(AuditModel):
    item = ForeignKeyProtect(Item)
    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    lotsman_document = ForeignKeyProtect(Lotsman_documents_hierarcy, verbose_name='Документ из Лоцмана', null=True, blank=True)

    objects = Item_documentManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс-таблица'
        constraints = [
            UniqueConstraint(fields=['item'], condition=Q(document=None) & Q(lotsman_document=None), name='Item_document_unique_constraint_0'),
            UniqueConstraint(fields=['document', 'item'], condition=Q(lotsman_document=None), name='Item_document_unique_constraint_1'),
            UniqueConstraint(fields=['item', 'lotsman_document'], condition=Q(document=None), name='Item_document_unique_constraint_2'),
            UniqueConstraint(fields=['document', 'item', 'lotsman_document'], name='Item_document_unique_constraint_3'),
        ]



from django.urls import path

from kaf_pas.ckk.views import materials

urlpatterns = [

    path('Materials/Fetch/', materials.Materials_Fetch),
    path('Materials/Add', materials.Materials_Add),
    path('Materials/Update', materials.Materials_Update),
    path('Materials/Remove', materials.Materials_Remove),
    path('Materials/Lookup/', materials.Materials_Lookup),
    path('Materials/Info/', materials.Materials_Info),

]



# Generated by Django 2.2.3 on 2019-07-03 22:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0123_material_added_values'),
    ]

    operations = [
        migrations.AlterField(
            model_name='material_type',
            name='code',
            field=models.CharField(max_length=255, unique=True),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-12 23:01

import bitfield.models
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0046_auto_20190610_0953'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document_attributes_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('value_str', models.TextField(db_index=True, verbose_name='Значение атрибута')),
                ('position_in_document', models.PositiveIntegerField()),
                ('section', isc_common.fields.name_field.NameField()),
                ('subsection', isc_common.fields.name_field.NameField()),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'), ('copied', 'Копированный')), db_index=True, default=1)),
            ],
            options={
                'verbose_name': 'Аттрибуты докуменнта',
                'db_table': 'kd_document_attrs_mview',
                'managed': False,
            },
        ),
        migrations.AlterUniqueTogether(
            name='document_attrs',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='document_attrs',
            name='attr_type',
        ),
        migrations.RemoveField(
            model_name='document_attrs',
            name='document',
        ),
        migrations.DeleteModel(
            name='Cdw_attrs',
        ),
        migrations.DeleteModel(
            name='Spw_attrs',
        ),
        migrations.CreateModel(
            name='Cdw_attrs',
            fields=[
                ('document_attrs_view_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='kd.Document_attrs_view')),
            ],
            options={
                'verbose_name': 'Чертеж',
                'managed': False,
            },
            bases=('kd.document_attrs_view',),
        ),
        migrations.CreateModel(
            name='Spw_attrs',
            fields=[
                ('document_attrs_view_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='kd.Document_attrs_view')),
            ],
            options={
                'verbose_name': 'Аттрибуты Спецификаций',
                'managed': False,
            },
            bases=('kd.document_attrs_view',),
        ),
        migrations.DeleteModel(
            name='Document_attrs',
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_USER

urlpatterns = [

    path('Document_attributes_SPC_CLM_USER/Fetch/', document_attributes_SPC_CLM_USER.Document_attributes_SPC_CLM_USER__Fetch),
    path('Document_attributes_SPC_CLM_USER/Add', document_attributes_SPC_CLM_USER.Document_attributes_SPC_CLM_USER__Add),
    path('Document_attributes_SPC_CLM_USER/Update', document_attributes_SPC_CLM_USER.Document_attributes_SPC_CLM_USER__Update),
    path('Document_attributes_SPC_CLM_USER/Remove', document_attributes_SPC_CLM_USER.Document_attributes_SPC_CLM_USER__Remove),
    path('Document_attributes_SPC_CLM_USER/Lookup/', document_attributes_SPC_CLM_USER.Document_attributes_SPC_CLM_USER__Lookup),
    path('Document_attributes_SPC_CLM_USER/Info/', document_attributes_SPC_CLM_USER.Document_attributes_SPC_CLM_USER__Info),
    path('Document_attributes_SPC_CLM_USER/Copy', document_attributes_SPC_CLM_USER.Document_attributes_SPC_CLM_USER__Copy),

]



# Generated by Django 2.2.2 on 2019-06-24 13:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0048_auto_20190613_1526'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='document_attributes_view',
            table='kd_document_attrs_view',
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-17 16:54

import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
        ('production', '0094_status_launch_props'),
    ]

    operations = [
        migrations.CreateModel(
            name='Launch_item_refs',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('child', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='launch_child', to='production.Launch_items')),
                ('item_refs', isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ckk.Item_refs')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='launch_parent', to='production.Launch_items')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-27 20:32

import bitfield.models
from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0004_auto_20190327_1141'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='item_line',
            options={'ordering': ['section'], 'verbose_name': 'Строка состава изделия'},
        ),
        migrations.RemoveField(
            model_name='item',
            name='document',
        ),
        migrations.RemoveField(
            model_name='item_line',
            name='item_child',
        ),
        migrations.RemoveField(
            model_name='item_line',
            name='item_parent',
        ),
        migrations.AddField(
            model_name='item_line',
            name='isFolder',
            field=models.BooleanField(default=False, verbose_name='Подраздел'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='parent',
            field=models.ManyToManyField(blank=True, null=True, to='ckk.Item_line'),
        ),
        migrations.AlterField(
            model_name='item',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('relevant1', 'Актуальность1')), default=1),
        ),
    ]



import logging

from django.db.models import PositiveIntegerField, DateTimeField, BooleanField

from isc_common.bit import IsBitOff
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import BaseRefHierarcy
from kaf_pas.sales.models.precent import Precent
from kaf_pas.sales.models.precent_dogovors import Precent_dogovors
from kaf_pas.sales.models.precent_items import Precent_items
from kaf_pas.sales.models.status_demand import Status_demand

logger = logging.getLogger(__name__)


class Demand_viewQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    pass


class Demand_viewManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'parent': record.parent.id if record.parent else None,
            'date': record.date,
            'date_sign': record.precent.date_sign,
            'qty': record.qty,
            'launch_qty': record.launch_qty,
            'tail_qty': record.tail_qty,
            'parent_id': record.parent.id if record.parent else None,

            'precent_id': record.precent.id,
            'precent__code': record.precent.code,
            'precent__date': record.precent.date,

            'precent__precent_type_id': record.precent.precent_type.id,
            'precent__precent_type__name': record.precent.precent_type.name,

            'precent_dogovor_id': record.precent_dogovor.id,
            'precent_dogovor__code': record.precent_dogovor.dogovor.code,
            'precent_dogovor__name': record.precent_dogovor.dogovor.name,
            'precent_dogovor__date': record.precent_dogovor.dogovor.date,

            'precent_dogovor__dogovor__customer__name': record.precent_dogovor.dogovor.customer.name,

            'precent_item_id': record.precent_item.id,
            'precent_item__STMP_1_id': record.precent_item.item.STMP_1.id if record.precent_item.item.STMP_1 else None,
            'precent_item__STMP_1__value_str': record.precent_item.item.STMP_1.value_str if record.precent_item.item.STMP_1 else None,
            'precent_item__STMP_2_id': record.precent_item.item.STMP_2.id if record.precent_item.item.STMP_2 else None,
            'precent_item__STMP_2__value_str': record.precent_item.item.STMP_2.value_str if record.precent_item.item.STMP_2 else None,

            'status_id': record.status.id,
            'status__code': record.status.code,
            'status__name': record.status.name,

            'isFolder': record.isFolder,
            'enabled': IsBitOff(record.status.props, 0),

            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Demand_viewQuerySet(self.model, using=self._db)


class Demand_view(BaseRefHierarcy):
    date = DateTimeField()
    precent = ForeignKeyCascade(Precent)
    precent_dogovor = ForeignKeyCascade(Precent_dogovors)
    precent_item = ForeignKeyCascade(Precent_items)
    status = ForeignKeyProtect(Status_demand)
    qty = PositiveIntegerField()
    launch_qty = PositiveIntegerField()
    tail_qty = PositiveIntegerField()
    isFolder = BooleanField()

    objects = Demand_viewManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Заказы на продажу'
        db_table = 'sales_demand_view'
        managed = False



import logging

from bitfield import BitField

from clndr.models.calendars import Calendars
from isc_common.fields.related import ForeignKeyProtect
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager
from isc_common.number import DelProps

logger = logging.getLogger(__name__)


class LocationsQuerySet(CommonManagetWithLookUpFieldsQuerySet, BaseRefQuerySet):
    pass


class LocationsManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def props():
        return BitField(flags=(
            ('isWorkshop', 'Уровень цеха'),  # 1
        ), default=0, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'full_name': record.full_name,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,
            'calendar_id': record.calendar.id if record.calendar else None,
            'calendar__full_name': record.calendar.full_name if record.calendar else None,
            'editing': record.editing,
            'deliting': record.deliting,
            'isWorkshop': record.props.isWorkshop,
            'props': record.props,
        }
        return DelProps(res)

    def get_queryset(self):
        return LocationsQuerySet(self.model, using=self._db)


class Locations(BaseRefHierarcy):
    calendar = ForeignKeyProtect(Calendars, null=True, blank=True)
    props = LocationsManager.props()

    objects = LocationsManager()

    objects_tree = TreeAuditModelManager()

    def _get_calendar(self, parent):
        if parent:
            if parent.calendar:
                return parent.calendar
            else:
                return self._get_calendar(parent=parent.parent)
        else:
            return None

    @property
    def get_calendar(self):
        if self.calendar:
            return self.calendar

        return self._get_calendar(parent=self.parent)

    def __repr__(self):
        return self.full_name

    def __str__(self):
        return f"ID: {self.id}, code: {self.code}, name: {self.name}, full_name: {self.full_name}, description: {self.description}"

    class Meta:
        verbose_name = 'Место положения'



# Generated by Django 3.0.4 on 2020-03-29 03:29

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0145_auto_20200326_1924'),
    ]

    operations = [
        migrations.AlterField(
            model_name='uploades',
            name='path',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Pathes'),
        ),
        migrations.AlterField(
            model_name='uploades_documents',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Documents'),
        ),
        migrations.AlterField(
            model_name='uploades_documents',
            name='upload',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Uploades'),
        ),
    ]



import logging

from django.core.management import BaseCommand

from isc_common import setAttr
from kaf_pas.production.models.launches import LaunchesManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование запуск"

    def add_arguments(self, parser):
        # parser.add_argument('--demand_id', type=int)
        parser.add_argument('--user_id', type=int)

    def handle(self, *args, **options):
        data = {'demand__date': '2019-09-25T00:00:00.000', 'demand__code': '19/407-1', 'code': '1', 'date': '2019-11-17', 'qty': 2, 'demand_id': 38, 'description': '11111111'}
        # demand_id = options.get('demand_id')
        user_id = options.get('user_id')
        setAttr(data, 'user_id', user_id)
        print(f'data: {data}')

        LaunchesManager.make_launch(data=data)

        print('Done')



import logging

from django.db.models import DecimalField, TextField
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.auth.models.user import User
from isc_common.fields.code_field import ColorField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from isc_common.models.standard_colors import Standard_colors
from isc_common.number import DelProps
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.locations import Locations

logger = logging.getLogger(__name__)


class Tmp_bufferQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    pass


class Tmp_bufferManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'location_id': record.location.id,
            'location__code': record.location.code,
            'location__name': record.location.name,
            'location__full_name': record.location.full_name,
            'item_id': record.item.id,
            'item__STMP_1_id': record.item.STMP_1.id if record.item.STMP_1 else None,
            'item__STMP_1__value_str': record.item.STMP_1.value_str if record.item.STMP_1 else None,
            'item__STMP_2_id': record.item.STMP_2.id if record.item.STMP_2 else None,
            'item__STMP_2__value_str': record.item.STMP_2.value_str if record.item.STMP_2 else None,
            'edizm_id': record.edizm.id,
            'edizm__code': record.edizm.code,
            'edizm__name': record.edizm.name,
            'editing': record.editing,
            'deliting': record.deliting,
            'description': record.description,
            'value': record.value,
            'color_id': record.color.id if record.color else None,
            'color__color': record.color.color if record.color else None,
            'color__name': record.color.name if record.color else None,
            'value_off': record.value_off,
        }
        return res

    def createFromRequest(self, request, removed=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'dataSource')
        delAttr(_data, 'operationType')
        delAttr(_data, 'textMatchStyle')
        delAttr(_data, 'form')
        _data.setdefault('user_id', request.user_id)
        self._remove_prop(_data, removed)

        res = super().create(**_data)
        try:
            full_name = res.full_name
        except:
            full_name = None
        res = model_to_dict(res)
        if full_name:
            setAttr(res, 'full_name', full_name)

        setAttr(res, 'isFolder', False)
        data.update(DelProps(res))
        return data

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        _data = data.copy()
        if _data.get('value_off'):
            if _data.get('value') < _data.get('value_off'):
                raise Exception(f'Превышен лимит.')

        delAttr(_data, 'id')
        delAttr(_data, 'item')
        delAttr(_data, 'location')
        delAttr(_data, 'user')
        delAttr(_data, 'edizm')
        res, _ = super().update_or_create(id=request.get_id(), defaults=_data)

        _data.setdefault('id', res.id)
        return _data

    def get_queryset(self):
        return Tmp_bufferQuerySet(self.model, using=self._db)


class Tmp_buffer(AuditModel):
    item = ForeignKeyProtect(Item)
    location = ForeignKeyProtect(Locations)
    user = ForeignKeyProtect(User)
    edizm = ForeignKeyProtect(Ed_izm)
    value = DecimalField(max_digits=19, decimal_places=4)
    value_off = DecimalField(max_digits=19, decimal_places=4, null=True, blank=True)
    description = TextField(null=True, blank=True)
    color = ForeignKeyProtect(Standard_colors, null=True, blank=True)

    objects = Tmp_bufferManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Временное хранилищи данных для буферов'



# Generated by Django 3.0.3 on 2020-02-27 14:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0216_auto_20200227_1404'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations_template_detail',
            name='description',
            field=models.TextField(blank=True, default=None, null=True),
        ),
    ]



import logging
import os

from django.db.models import TextField, BigAutoField, DateTimeField, BooleanField, Model, CharField
from django.utils import timezone

from isc_common.fields.related import ForeignKeyProtect
from isc_common.number import DelProps
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.pathes import Pathes, PathesManager

logger = logging.getLogger(__name__)


class Pathes_viewManager(PathesManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "path": record.path,
            "virt_path": record.virt_path,
            "parent_id": record.parent_id,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
            "isFolder": record.isFolder,
            "isUpload": record.isUpload,
            "drive": record.drive,
            "props": record.props,
            "enable_upload": record.props.enable_upload,
            "attr_type_id": record.attr_type.id if record.attr_type else None,
            "attr_type__code": record.attr_type.code if record.attr_type else None,
            "attr_type__name": record.attr_type.name if record.attr_type else None,
        }
        return DelProps(res)

class Pathes_view(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    deleted_at = DateTimeField(verbose_name="Дата мягкого удаления", null=True, blank=True, db_index=True)
    editing = BooleanField(verbose_name="Возможность редактирования", default=True)
    deliting = BooleanField(verbose_name="Возможность удаления", default=True)
    lastmodified = DateTimeField(verbose_name='Последнее обновление', editable=False, db_index=True, default=timezone.now)

    isFolder = BooleanField(null=True, blank=True, )
    isUpload = BooleanField(null=True, blank=True, )

    drive = CharField(max_length=10, null=True, blank=True)
    path = TextField(verbose_name="Путь")
    virt_path = TextField(verbose_name="Мнимый путь", null=True, blank=True)
    parent = ForeignKeyProtect("self", null=True, blank=True)
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Атрибут', null=True, blank=True)

    props = PathesManager.props()

    @property
    def absolute_path(self):
        def get_parent(item_tuple):
            if item_tuple[0].parent:
                res = Pathes.objects.get(id=item_tuple[0].parent.id)
                res = (res, f"{res.path}/{item_tuple[1]}")
                return get_parent(res)
            else:
                return item_tuple

        if self.parent:
            res = get_parent((self, self.path))
            return f'{os.altsep}{res[1]}'
        else:
            return f'{os.altsep}{self.path}'

    def __str__(self):
        return f"{self.absolute_path}"

    objects = Pathes_viewManager()

    class Meta:
        db_table = 'kd_pathes_view'
        managed = False
        verbose_name = 'Пути нахождения документов'



from django.urls import path

from kaf_pas.ckk.views import ed_izm

urlpatterns = [

    path('Ed_izm/Fetch/', ed_izm.Ed_izm_Fetch),
    path('Ed_izm/Add', ed_izm.Ed_izm_Add),
    path('Ed_izm/Update', ed_izm.Ed_izm_Update),
    path('Ed_izm/Remove', ed_izm.Ed_izm_Remove),
    path('Ed_izm/Lookup/', ed_izm.Ed_izm_Lookup),
    path('Ed_izm/Info/', ed_izm.Ed_izm_Info),

]



# Generated by Django 3.0.3 on 2020-02-27 08:59

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0213_auto_20200227_0819'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations_template',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Operations_template'),
        ),
        # migrations.AddConstraint(
        #     model_name='resource',
        #     constraint=models.UniqueConstraint(condition=models.Q(('code', None), ('name', None)), fields=('location',), name='Resource_unique_constraint_0'),
        # ),
        # migrations.AddConstraint(
        #     model_name='resource',
        #     constraint=models.UniqueConstraint(condition=models.Q(code=None), fields=('location', 'name'), name='Resource_unique_constraint_1'),
        # ),
    ]



# Generated by Django 2.2.3 on 2019-07-03 11:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('refs', '0004_type_param_values'),
        ('ckk', '0121_auto_20190703_0506'),
    ]

    operations = [
        migrations.RenameField(
            model_name='material_params',
            old_name='meterial_type',
            new_name='materials_type',
        ),
        migrations.RenameField(
            model_name='materials',
            old_name='type',
            new_name='materials_type',
        ),
        migrations.AlterUniqueTogether(
            name='material_params',
            unique_together={('materials_type', 'param_type')},
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import item_image_refs

urlpatterns = [

    path('Item_image_refs/Fetch/', item_image_refs.Item_image_refs_Fetch),
    path('Item_image_refs/Add', item_image_refs.Item_image_refs_Add),
    path('Item_image_refs/Update', item_image_refs.Item_image_refs_Update),
    path('Item_image_refs/Remove', item_image_refs.Item_image_refs_Remove),
    path('Item_image_refs/Lookup/', item_image_refs.Item_image_refs_Lookup),
    path('Item_image_refs/Info/', item_image_refs.Item_image_refs_Info),
    path('Item_image_refs/Copy', item_image_refs.Item_image_refs_Copy),
]



import logging

from django.db.models import PositiveIntegerField

from isc_common import setAttr, delAttr
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsQuerySet, CommonManagetWithLookUpFieldsManager
from isc_common.models.audit import AuditModel
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.material_askon import Material_askon
from kaf_pas.ckk.models.materials import Materials
from kaf_pas.sales.models.precent_items import Precent_items

logger = logging.getLogger(__name__)


class Precent_materialsQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        if kwargs.get('material'):
            setAttr(kwargs, 'material_id', kwargs.get('material').id)
            delAttr(kwargs, 'material')

        if kwargs.get('material_askon'):
            setAttr(kwargs, 'material_askon_id', kwargs.get('material_askon').id)
            delAttr(kwargs, 'material_askon')

        if not kwargs.get('material_id') and not kwargs.get('material_askon_id'):
            raise Exception('Необходим хотябы один выбранный параметр.')

        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Precent_materialsManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            'material_askon_id': record.material_askon.id if record.material_askon else None,
            'material_askon__field0': record.material_askon.field0 if record.material_askon else None,

            'material_id': record.material.id if record.material else None,
            'material__name': record.material.name if record.material else None,

            'complex_name': record.complex_name,
            'complex_gost': record.complex_gost,

            'edizm_id': record.edizm.id,
            'edizm__code': record.edizm.code,
            'edizm__name': record.edizm.name,

            'qty': record.qty,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def get_queryset(self):
        return Precent_materialsQuerySet(self.model, using=self._db)


class Precent_materials(AuditModel):
    precent_item = ForeignKeyCascade(Precent_items)
    material = ForeignKeyProtect(Materials, null=True, blank=True)
    material_askon = ForeignKeyProtect(Material_askon, null=True, blank=True)
    edizm = ForeignKeyProtect(Ed_izm, default=None)
    qty = PositiveIntegerField()

    @property
    def complex_name(self):
        if self.material:
            return f'{self.material.materials_type.full_name}{self.material.full_name}'
        else:
            return None

    @property
    def complex_gost(self):
        if self.material:
            if self.material.materials_type.gost:
                return f'{self.material.materials_type.gost} / {self.material.gost}'
            else:
                return self.material.gost
        else:
            return None

    objects = Precent_materialsManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Комплектация'



from django.urls import path

from kaf_pas.production.views import operations_template_material

urlpatterns = [

    path('Operations_template_material/Fetch/', operations_template_material.Operations_template_material_Fetch),
    path('Operations_template_material/Add', operations_template_material.Operations_template_material_Add),
    path('Operations_template_material/Update', operations_template_material.Operations_template_material_Update),
    path('Operations_template_material/Remove', operations_template_material.Operations_template_material_Remove),
    path('Operations_template_material/Lookup/', operations_template_material.Operations_template_material_Lookup),
    path('Operations_template_material/Info/', operations_template_material.Operations_template_material_Info),
    path('Operations_template_material/Copy', operations_template_material.Operations_template_material_Copy),

]



from django.urls import path

from kaf_pas.ckk.views import item_flat_view

urlpatterns = [

    path('Item_flat_view/Fetch/', item_flat_view.Item_flat_view_Fetch),
    path('Item_flat_view/FetchPlan/', item_flat_view.Item_flat_view_FetchPlan),
    path('Item_flat_view/Add', item_flat_view.Item_flat_view_Add),
    path('Item_flat_view/Update', item_flat_view.Item_flat_view_Update),
    path('Item_flat_view/Remove', item_flat_view.Item_flat_view_Remove),
    path('Item_flat_view/Lookup/', item_flat_view.Item_flat_view_Lookup),
    path('Item_flat_view/Info/', item_flat_view.Item_flat_view_Info),
    path('Item_flat_view/InfoPlan/', item_flat_view.Item_flat_view_InfoPlan),
    path('Item_flat_view/Copy', item_flat_view.Item_flat_view_Copy),

]



from django.urls import path

from kaf_pas.ckk.views import material_type

urlpatterns = [

    path('Material_type/Fetch/', material_type.Material_type_Fetch),
    path('Material_type/Add', material_type.Material_type_Add),
    path('Material_type/Update', material_type.Material_type_Update),
    path('Material_type/Remove', material_type.Material_type_Remove),
    path('Material_type/Lookup/', material_type.Material_type_Lookup),
    path('Material_type/Info/', material_type.Material_type_Info),
    path('Material_type/Copy', material_type.Material_type_Copy),
    path('Material_type/CopyParams', material_type.Material_type_CopyParams),

]



# Generated by Django 2.2.8 on 2019-12-06 15:54

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0029_status_operation_types'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='planing.Status_operation_types'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 11:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0173_auto_20200208_1435'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='launch_operations_item',
            name='Launch_operations_item_unique_constraint_0',
        ),
        migrations.RemoveConstraint(
            model_name='launch_operations_item',
            name='Launch_operations_item_unique_constraint_1',
        ),
        migrations.AddConstraint(
            model_name='launch_operations_item',
            constraint=models.UniqueConstraint(condition=models.Q(num=None), fields=('item', 'launch', 'operation'), name='Launch_operations_item_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='launch_operations_item',
            constraint=models.UniqueConstraint(fields=('item', 'launch', 'num', 'operation'), name='Launch_operations_item_unique_constraint_1'),
        ),
    ]



import logging

from django.db import ProgrammingError

logger = logging.getLogger(__name__)


class Prod_Status_Stack:
    def get_first_item_of_tuple(self, tp):
        res, _ = tp
        return res

    def __init__(self):
        from kaf_pas.ckk.models.attr_type import AttrManager

        try:
            from kaf_pas.production.models.status_launch import Status_launch

            self.FORMIROVANIE = self.get_first_item_of_tuple(Status_launch.objects.update_or_create(code='formirovanie', defaults=dict(
                name='Формирование',
                editing=False,
                deliting=False,
            )))

            self.ROUTMADE = self.get_first_item_of_tuple(Status_launch.objects.update_or_create(code='route_made', defaults=dict(
                name='Выполнена маршрутизация',
                editing=False,
                deliting=False,
            )))

            self.IN_PRODUCTION = self.get_first_item_of_tuple(Status_launch.objects.update_or_create(code='in_production', defaults=dict(
                name='Сформированы заказы на производство',
                editing=False,
                deliting=False,
            )))

            self.CLOSED = self.get_first_item_of_tuple(Status_launch.objects.update_or_create(code='closed', defaults=dict(
                name='Закрыт',
                editing=False,
                deliting=False,
            )))

            AttrManager.get_or_create_attr(attr_codes='operations.paremetrs.color', attr_names='Операции.Параметры.Цвет')

        except ProgrammingError as ex:
            logger.warning(ex)



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_NAME

urlpatterns = [

    path('Document_attributes_SPC_CLM_NAME/Fetch/', document_attributes_SPC_CLM_NAME.Document_attributes_SPC_CLM_NAME_Fetch),
    path('Document_attributes_SPC_CLM_NAME/Add', document_attributes_SPC_CLM_NAME.Document_attributes_SPC_CLM_NAME_Add),
    path('Document_attributes_SPC_CLM_NAME/Update', document_attributes_SPC_CLM_NAME.Document_attributes_SPC_CLM_NAME_Update),
    path('Document_attributes_SPC_CLM_NAME/Remove', document_attributes_SPC_CLM_NAME.Document_attributes_SPC_CLM_NAME_Remove),
    path('Document_attributes_SPC_CLM_NAME/Lookup/', document_attributes_SPC_CLM_NAME.Document_attributes_SPC_CLM_NAME_Lookup),
    path('Document_attributes_SPC_CLM_NAME/Info/', document_attributes_SPC_CLM_NAME.Document_attributes_SPC_CLM_NAME_Info),
    path('Document_attributes_SPC_CLM_NAME/Copy', document_attributes_SPC_CLM_NAME.Document_attributes_SPC_CLM_NAME_Copy),

]



# Generated by Django 2.2.4 on 2019-08-30 17:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0150_status_code_owner'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='status',
            unique_together=set(),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-21 18:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0008_auto_20190521_1853'),
        ('sales', '0014_auto_20190521_1517'),
    ]

    operations = [
    ]



# Generated by Django 2.2.5 on 2019-09-16 17:59

from django.db import migrations
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0011_user_add_info_color'),
    ]

    operations = [
        migrations.AddField(
            model_name='user_add_info',
            name='phones',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
    ]



# Generated by Django 2.2.2 on 2019-07-01 17:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0114_material_type_parent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='material_type',
            name='code',
            field=models.CharField(default=None, max_length=255),
        ),
        migrations.AlterField(
            model_name='material_type',
            name='name',
            field=models.CharField(default=None, max_length=255),
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import DateTimeField
from django.forms import model_to_dict
from django.utils import timezone

from crypto.models.crypto_file import Crypto_file, CryptoManager, CryptoQuerySet
from isc_common import delAttr, setAttr
from isc_common.fields.code_field import CodeStrictField
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseHierarcy
from kaf_pas.sales.models.customer import Customer
from kaf_pas.sales.models.dogovor_types import Dogovor_types
from kaf_pas.sales.models.status_dogovor import StatusDogovor

logger = logging.getLogger(__name__)


class DogovorsQuerySet(CryptoQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class DogovorsManager(CryptoManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'name': record.name,
            'full_name': record.full_name,
            'date': record.date,
            'date_sign': record.date_sign,
            'description': record.description,

            'status_id': record.status.id,
            'status__code': record.status.code,
            'status__name': record.status.name,

            'type_id': record.type.id,
            'type__code': record.type.code,
            'type__name': record.type.name,

            'customer_id': record.customer.id,
            'customer__inn': record.customer.inn,
            'customer__kpp': record.customer.kpp,
            'customer__name': record.customer.name,
            'customer__full_name': record.customer.full_name,
            'customer__description': record.customer.description,

            'format': record.format,
            'mime_type': record.mime_type,
            'size': record.size if not str(record.size).startswith('.') else str(record.size).replace('.', '0.'),
            'real_name': record.real_name,

            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
            'parent_id': record.parent.id if record.parent else None,
        }
        return res

    def createFromRequest(self, request, printRequest=False, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'form')
        if data.get('id'):
            delAttr(_data, 'id')
            res = super().filter(id=data.get('id')).update(**_data)
        else:
            res = super().create(**_data)
            res = model_to_dict(res)
            delAttr(res, 'attfile')
            setAttr(res, 'props', res.get( 'props')._value)
            data.update(res)
        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'form')
        delAttr(_data, 'isFolder')
        delAttr(_data, 'full_name')
        delAttr(_data, 'enabled')
        super().filter(id=request.get_id()).update(**_data)
        return data

    def get_queryset(self):
        return DogovorsQuerySet(self.model, using=self._db)


class Dogovors(Crypto_file, BaseHierarcy):
    name = CodeStrictField()
    description = DescriptionField()

    date = DateTimeField(verbose_name='Дата', db_index=True, default=timezone.now)
    date_sign = DateTimeField(verbose_name='Дата подписания', db_index=True, null=True, blank=True)

    status = ForeignKeyProtect(StatusDogovor)
    customer = ForeignKeyProtect(Customer)
    type = ForeignKeyProtect(Dogovor_types)

    props = BitField(flags=(
        ('real', 'Реальный документ'),
    ), default=1, db_index=True)

    objects = DogovorsManager()

    def __str__(self):
        return f'{self.id}'

    class Meta:
        verbose_name = 'Договора'



from django.urls import path

from kaf_pas.ckk.views import attr_type

urlpatterns = [

    path('Attr_type/Fetch/', attr_type.Attr_type_Fetch),
    path('Attr_type/Add', attr_type.Attr_type_Add),
    path('Attr_type/Update', attr_type.Attr_type_Update),
    path('Attr_type/Remove', attr_type.Attr_type_Remove),
    path('Attr_type/Lookup/', attr_type.Attr_type_Lookup),
    path('Attr_type/Info/', attr_type.Attr_type_Info),

]



# Generated by Django 2.2.3 on 2019-07-20 13:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0071_auto_20190720_1313'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='document_attr_cross',
            unique_together={('document', 'position_in_document')},
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operation_color

urlpatterns = [

    path('Operation_color/Fetch/', operation_color.Operation_color_Fetch),
    path('Operation_color/Add', operation_color.Operation_color_Add),
    path('Operation_color/Update', operation_color.Operation_color_Update),
    path('Operation_color/Remove', operation_color.Operation_color_Remove),
    path('Operation_color/Lookup/', operation_color.Operation_color_Lookup),
    path('Operation_color/Info/', operation_color.Operation_color_Info),
    path('Operation_color/Copy', operation_color.Operation_color_Copy),

]



from django.urls import path

from kaf_pas.production.views import operation_def_resources

urlpatterns = [

    path('Operation_def_resources/Fetch/', operation_def_resources.Operation_def_resources_Fetch),
    path('Operation_def_resources/Add', operation_def_resources.Operation_def_resources_Add),
    path('Operation_def_resources/Update', operation_def_resources.Operation_def_resources_Update),
    path('Operation_def_resources/AllUpdate', operation_def_resources.Operation_def_resources_AllUpdate),
    path('Operation_def_resources/Remove', operation_def_resources.Operation_def_resources_Remove),
    path('Operation_def_resources/Lookup/', operation_def_resources.Operation_def_resources_Lookup),
    path('Operation_def_resources/Info/', operation_def_resources.Operation_def_resources_Info),
    path('Operation_def_resources/Copy', operation_def_resources.Operation_def_resources_Copy),

]



from django.db import IntegrityError
from django.test import TestCase

from isc_common.auth.models.user import User
from isc_common.models.audit import IsBlankStr, NotUnique

from kaf_pas.ckk.models.attr_type import Attr_type


class Test_Attr_type(TestCase):
    def setUp(self):
        self.code = "code"
        self.name = "name"
        self.description = "parent"

    def test_1(self):
        attr_type = Attr_type.objects.create(code=self.code, name=self.name, description= self.description)
        attr_type = Attr_type.objects.get(pk=attr_type.id)
        self.assertEquals(attr_type.code, self.code)
        self.assertEquals(attr_type.name, self.name)
        self.assertEquals(attr_type.description, self.description)
        self.assertEquals(attr_type.parent, None)

        attr_type1 = attr_type
        self.code += "1"
        attr_type = Attr_type.objects.create(code=self.code, name=self.name, parent=attr_type1)
        attr_type = Attr_type.objects.get(pk=attr_type.id)
        self.assertEquals(attr_type.description, None)
        self.assertEquals(attr_type.name, self.name)
        self.assertEquals(attr_type.parent, attr_type1)

    def test_2(self):
        # with self.assertRaises(IntegrityError):
        Attr_type.objects.create(code=self.code, name=self.name)
        with self.assertRaises(IntegrityError):
            Attr_type.objects.create(name=self.name)

    def test_4(self):
        Attr_type.objects.create(code=self.code, name=self.name)
        with self.assertRaises(IntegrityError):
            Attr_type.objects.create(code=self.code, name=self.name)

    def tets_column_types(self):
        attr = Attr_type.objects.get_attr('SPC_CLM_FORMAT')
        self.assertEquals(attr.id, 4)
        attr = Attr_type.objects.get_attr('SPC_CLM_ZONE')
        self.assertEquals(attr.id, 5)
        attr = Attr_type.objects.get_attr('SPC_CLM_POS')
        self.assertEquals(attr.id, 6)
        attr = Attr_type.objects.get_attr('SPC_CLM_MARK')
        self.assertEquals(attr.id, 7)
        attr = Attr_type.objects.get_attr('SPC_CLM_NAME')
        self.assertEquals(attr.id, 8)
        attr = Attr_type.objects.get_attr('SPC_CLM_COUNT')
        self.assertEquals(attr.id, 9)
        attr = Attr_type.objects.get_attr('SPC_CLM_NOTE')
        self.assertEquals(attr.id, 10)
        attr = Attr_type.objects.get_attr('SPC_CLM_MASSA')
        self.assertEquals(attr.id, 11)
        attr = Attr_type.objects.get_attr('SPC_CLM_MATERIAL')
        self.assertEquals(attr.id, 12)
        attr = Attr_type.objects.get_attr('SPC_CLM_USER')
        self.assertEquals(attr.id, 13)
        attr = Attr_type.objects.get_attr('SPC_CLM_KOD')
        self.assertEquals(attr.id, 14)
        attr = Attr_type.objects.get_attr('SPC_CLM_FACTORY')
        self.assertEquals(attr.id, 15)



# Generated by Django 2.2.1 on 2019-05-29 11:12

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0031_auto_20190529_1111'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='category',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operation_categories'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-18 11:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0061_auto_20190918_1459'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dogovors',
            name='date',
            field=models.DateTimeField(db_index=True, default=None, verbose_name='Дата'),
        ),
        migrations.AlterField(
            model_name='precent',
            name='date',
            field=models.DateTimeField(db_index=True, default=None, verbose_name='Дата'),
        ),
    ]



import logging

from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.number import DelProps
from kaf_pas.planing.models.operations import Operations, OperationsManager
from kaf_pas.production.models.launches import Launches

logger = logging.getLogger(__name__)


class Operation_launchesQuerySet(AuditQuerySet):
    pass


class Operation_launchesManager(AuditManager):

    def makeProdOrderFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        _data.setdefault('user', request.user)
        OperationsManager.make_production_order(data=_data)
        return data

    def deleteProdOrderFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        _data.setdefault('user', request.user)
        OperationsManager.delete_production_order(data=_data)
        return data

    def reCalcRoutesFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        _data.setdefault('user', request.user)
        OperationsManager.make_routing(data=_data)
        return data

    def cleanRoutesFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        data.setdefault('user', request.user)
        res = OperationsManager.clean_routing(data=data)
        return DelProps(res)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_launchesQuerySet(self.model, using=self._db)


class Operation_launches(AuditModel):
    operation = ForeignKeyCascade(Operations, related_name='planing_operation_launch')
    launch = ForeignKeyCascade(Launches, related_name='planing_resource_launch')

    objects = Operation_launchesManager()

    def __str__(self):
        return f"ID:{self.id}, operation: [{self.operation}], launch: [{self.launch}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('operation', 'launch'),)



# Generated by Django 2.2.8 on 2019-12-25 20:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system', '0004_contants_parent'),
    ]

    operations = [
        migrations.AddField(
            model_name='contants',
            name='value',
            field=models.TextField(blank=True, null=True),
        ),
    ]



import logging

from kaf_pas.kd.models.documents import Documents, DocumentManager, DocumentQuerySet

logger = logging.getLogger(__name__)


class PdfQuerySet(DocumentQuerySet):
    ...


class PdfManager(DocumentManager):
    def get_queryset(self):
        return PdfQuerySet(self.model, using=self._db).filter(attr_type__code='KD_PDF')


class Pdfs(Documents):
    objects = PdfManager()

    class Meta:
        verbose_name = 'Бумажные Чертежи'
        proxy = True



import requests
from django.conf import settings
from django.http import HttpResponseRedirect

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException, JsonWSResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.documents_thumb import Documents_thumb, Documents_thumbManager


@JsonResponseWithException()
def Documents_thumb_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Documents_thumb.objects.
                select_related('document', 'lotsman_document').
                get_range_rows1(
                request=request,
                function=Documents_thumbManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_thumb_Add(request):
    return JsonResponse(DSResponseAdd(data=Documents_thumb.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_thumb_Update(request):
    return JsonResponse(DSResponseUpdate(data=Documents_thumb.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonWSResponseWithException()
def Documents_thumb_Remove(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)

    _transaction = _request.json.get('transaction')
    ids = []
    if _transaction:
        for operation in _transaction.get('operations'):
            data = operation.get('data')
            for id in data.get('ids'):
                ids.append((id, None))
    else:
        data = _request.json.get('data')
        for id in data.get('ids'):
            ids.append((id, None))

    for id in ids:
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10

        thumb = Documents_thumb.objects.get(id=id[0])
        for thumb10 in Documents_thumb10.objects.filter(path=thumb.path):
            Item_image_refs.objects.filter(thumb10=thumb10).delete()

        Item_image_refs.objects.filter(thumb=thumb).delete()

    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/DocumentsThumb/Remove', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_thumb_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Documents_thumb.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_thumb_Info(request):
    return JsonResponse(DSResponse(request=request, data=Documents_thumb.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2 on 2019-04-13 12:15

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0006_auto_20190413_1116'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documents',
            name='props',
            field=bitfield.models.BitField(('relevant', 'Актуальность'), default=1),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.demand import Demand
from kaf_pas.sales.models.demand_view import Demand_view, Demand_viewManager


@JsonResponseWithException()
def Demand_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Demand_view.objects.
                select_related('precent', 'precent_dogovor', 'status', 'precent_item').
                get_range_rows1(
                request=request,
                function=Demand_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_Add(request):
    return JsonResponse(DSResponseAdd(data=Demand.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_Update(request):
    return JsonResponse(DSResponseUpdate(data=Demand.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Demand.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Demand.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_Info(request):
    return JsonResponse(DSResponse(request=request, data=Demand.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Demand_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Demand.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from django.urls import path

from kaf_pas.sales.views import customer

urlpatterns = [

    path('Customer/Fetch/', customer.Customer_Fetch),
    path('Customer/Add', customer.Customer_Add),
    path('Customer/Update', customer.Customer_Update),
    path('Customer/Remove', customer.Customer_Remove),
    path('Customer/Lookup/', customer.Customer_Lookup),
    path('Customer/Info/', customer.Customer_Info),

]



# Generated by Django 2.2.4 on 2019-08-08 13:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0080_remove_pathes_props'),
        ('ckk', '0139_item_version'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item',
            unique_together={('STMP_1', 'STMP_2', 'props', 'version')},
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-19 16:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0002_buffers'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='buffers',
            table='accounting_buffers_view',
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-22 17:45

from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0015_auto_20191116_1321'),
    ]

    operations = [
        migrations.CreateModel(
            name='User_agg_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('username', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('first_name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('last_name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('middle_name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('location_position', isc_common.fields.name_field.NameField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Пользователи',
                'db_table': 'user_agg_view',
                'managed': False,
            },
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-17 18:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0052_auto_20190822_2250'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='precent_item_line',
            name='item_line',
        ),
        migrations.RemoveField(
            model_name='precent_item_line',
            name='location',
        ),
        migrations.RemoveField(
            model_name='precent_item_line',
            name='precent',
        ),
        migrations.DeleteModel(
            name='Precent_item_line_view',
        ),
        migrations.DeleteModel(
            name='Precent_item_line',
        ),
    ]



# Generated by Django 3.0.4 on 2020-03-18 12:00

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0141_auto_20200222_1755'),
    ]

    operations = [
        migrations.AddField(
            model_name='pathes',
            name='props',
            field=bitfield.models.BitField((('enable_upload', 'Директория для закачки'),), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-29 14:13

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0105_auto_20191029_0729'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dogovors',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='sales.Dogovors'),
        ),
    ]



import logging

from django.core.management import BaseCommand

from isc_common.common.functions import delete_dbl_spaces
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_view import Item_view
from kaf_pas.kd.models.documents_ext import DocumentManagerExt

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    def handle(self, *args, **options):
        logger.info(self.help)

        print(delete_dbl_spaces('К4310М-243-1.10.20.660           СБ'))



from django.urls import path

from kaf_pas.sales.views import precent_materials

urlpatterns = [

    path('Precent_materials/Fetch/', precent_materials.Precent_materials_Fetch),
    path('Precent_materials/Add', precent_materials.Precent_materials_Add),
    path('Precent_materials/Update', precent_materials.Precent_materials_Update),
    path('Precent_materials/Remove', precent_materials.Precent_materials_Remove),
    path('Precent_materials/Lookup/', precent_materials.Precent_materials_Lookup),
    path('Precent_materials/Info/', precent_materials.Precent_materials_Info),

]



from django.urls import path

from kaf_pas.production.views import launch_item_refs

urlpatterns = [

    path('Launch_item_refs/Fetch/', launch_item_refs.Launch_item_refs_Fetch),
    path('Launch_item_refs/Add', launch_item_refs.Launch_item_refs_Add),
    path('Launch_item_refs/Update', launch_item_refs.Launch_item_refs_Update),
    path('Launch_item_refs/Remove', launch_item_refs.Launch_item_refs_Remove),
    path('Launch_item_refs/Lookup/', launch_item_refs.Launch_item_refs_Lookup),
    path('Launch_item_refs/Info/', launch_item_refs.Launch_item_refs_Info),
    path('Launch_item_refs/Copy', launch_item_refs.Launch_item_refs_Copy),

]



from django.urls import path

from kaf_pas.input.views import user_add_info

urlpatterns = [

    path('User_add_info/Fetch/', user_add_info.User_add_info_Fetch),
    path('User_add_info/Add', user_add_info.User_add_info_Add),
    path('User_add_info/Update', user_add_info.User_add_info_Update),
    path('User_add_info/Remove', user_add_info.User_add_info_Remove),
    path('User_add_info/Lookup/', user_add_info.User_add_info_Lookup),
    path('User_add_info/Info/', user_add_info.User_add_info_Info),

]



# Generated by Django 2.2.1 on 2019-05-29 06:09

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0065_auto_20190529_0609'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='parent',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='item_parent', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
    ]



import logging

from django.db import ProgrammingError

logger = logging.getLogger(__name__)


class Operation_Stack:
    def get_first_item_of_tuple(self, tp):
        res, _ = tp
        return res

    def __init__(self):
        from kaf_pas.production.models.resource import Resource

        try:
            from kaf_pas.production.models.operations import Operations
            self.NOOP = self.get_first_item_of_tuple(Operations.objects.update_or_create(code='00', defaults=dict(
                name='Пустая операция',
                editing=False,
                deliting=False,
            )))

            from kaf_pas.ckk.models.locations import Locations
            self.NO_WORKSHOP = self.get_first_item_of_tuple(Locations.objects.update_or_create(code='00', defaults=dict(
                name='Фиктивный цех',
                editing=False,
                deliting=False,
            )))

            self.NOT_UNDEFINED_WORKSHOP = lambda location: Resource.objects.get_or_create(
                location=location,
                code='none',
                defaults=dict(
                    name='Не определен'
                )
            )


        except ProgrammingError as ex:
            logger.warning(ex)



# Generated by Django 2.2.8 on 2019-12-07 10:31

from django.db import migrations

from kaf_pas.production.models.operations_item import Operations_itemManager


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0156_auto_20191207_0910'),
    ]

    operations = [
        migrations.RunPython(Operations_itemManager.refresh_num),
    ]



# Generated by Django 2.2.8 on 2019-12-07 23:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0043_operations_level'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_launch_operation_resources',
            name='launch_operation_resources',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operation_resources',
            name='operation',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operations_item',
            name='launch_operations_item',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operations_item',
            name='operation',
        ),
        migrations.DeleteModel(
            name='Operation_launch_operation_material',
        ),
        migrations.DeleteModel(
            name='Operation_launch_operation_resources',
        ),
        migrations.DeleteModel(
            name='Operation_launch_operations_item',
        ),
    ]



import logging

from django.db.models import UniqueConstraint, Q

from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager

from kaf_pas.ckk.models.item import Item
from kaf_pas.kd.models.documents_thumb import Documents_thumb
from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10

logger = logging.getLogger(__name__)


class Item_image_refsManager(AuditManager):
    @staticmethod
    def getRecord(record):
        res = {
            "id": record.thumb.id if record.thumb else None,
            "item_id": record.item.id,
            "path": record.thumb.path if record.thumb else None,
            "file_document_thumb_url": f'/logic/DocumentsThumb/Download/{record.thumb.id}/' if record.thumb else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def deleteFromRequest(self, request, removed=None, ):
        from django.db import transaction

        request = DSRequest(request=request)
        res = 0

        _transaction = request.json.get('transaction')
        if _transaction:
            with transaction.atomic():
                for operation in _transaction.get('operations'):
                    data = operation.get('data')
                    for id in data.get('ids'):
                        for item in data.get('items'):
                            res += Item_image_refs.objects.filter(item_id=item.get('id'), thumb_id=id).delete()[0]
        else:
            data = request.json.get('data')
            for id in data.get('ids'):
                for item in data.get('items'):
                    if item.get('child_id'):
                        item_id = item.get('child_id')
                    else:
                        item_id = item.get('id')
                    res += Item_image_refs.objects.filter(item_id=item_id, thumb_id=id).delete()[0]
        return res


class Item_image_refs(AuditModel):
    item = ForeignKeyCascade(Item)
    thumb = ForeignKeyCascade(Documents_thumb, null=True, blank=True)
    thumb10 = ForeignKeyCascade(Documents_thumb10, null=True, blank=True)

    objects = Item_image_refsManager()

    def __str__(self):
        return f"item: {self.item}, thumb: {self.thumb}, thumb10: {self.thumb10}"

    class Meta:
        constraints = [
            UniqueConstraint(fields=['item'], condition=Q(thumb10=None) & Q(thumb=None), name='Item_image_refs_unique_constraint_0'),
            UniqueConstraint(fields=['item', 'thumb'], condition=Q(thumb10=None), name='Item_image_refs_unique_constraint_1'),
            UniqueConstraint(fields=['item', 'thumb10'], condition=Q(thumb=None), name='Item_image_refs_unique_constraint_2'),
            UniqueConstraint(fields=['item', 'thumb', 'thumb10'], name='Item_image_refs_unique_constraint_3'),
        ]
        verbose_name = 'Кросс таблица на местоположения графических элементов'



# Generated by Django 2.2.6 on 2019-10-19 19:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0004_auto_20191019_1955'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operationsplan',
            name='quantity_minus',
            field=models.DecimalField(decimal_places=8, max_digits=20),
        ),
        migrations.AlterField(
            model_name='operationsplan',
            name='quantity_plus',
            field=models.DecimalField(decimal_places=8, max_digits=20),
        ),
    ]



import requests
from django.http import HttpResponseRedirect

from isc_common import ws
from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.pathes import Pathes, PathesManager
from django.conf import settings


@JsonResponseWithException()
def Pathes_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Pathes.objects.
                filter().
                get_range_rows1(
                request=request,
                function=PathesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Pathes_Add(request):
    return JsonResponse(DSResponseAdd(data=Pathes.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Pathes_Update(request):
    return JsonResponse(DSResponseUpdate(data=Pathes.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Pathes_Remove(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    ids = _request.get_tuple_ids()
    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Pathes/Remove', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Pathes_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Pathes.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Pathes_Info(request):
    return JsonResponse(DSResponse(request=request, data=Pathes.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.7 on 2019-11-18 11:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
        ('production', '0103_auto_20191118_1027'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='launch_item_line',
            unique_together={('parent', 'child', 'launch')},
        ),
        migrations.AlterUniqueTogether(
            name='launch_item_refs',
            unique_together={('parent', 'child', 'launch')},
        ),
    ]



import logging

from bitfield import BitField
from django.db import transaction

from isc_common.bit import TurnBitOn
from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefManager, BaseRefQuerySet
from isc_common.progress import managed_progress, ProgressDroped
from kaf_pas.production.models import progress_deleted

logger = logging.getLogger(__name__)


class AttrQuerySet(BaseRefQuerySet):
    pass


class AttrManager(BaseRefManager):
    def deleteFromRequest(self, request, removed=None, ):
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.document_attributes import Document_attributes
        from kaf_pas.kd.models.lotsman_document_attr_cross import Lotsman_document_attr_cross

        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        for id, mode in tuple_ids:
            if mode == 'hide':
                super().filter(id=id).soft_delete()
            else:
                cnt = Document_attributes.objects.filter(attr_type_id=id).count()
                with managed_progress(
                        id=f'delete_{id}',
                        qty=Document_attributes.objects.filter(attr_type_id=id).count(),
                        user=request.user,
                        title='Удаление атрибута(ов)',
                        props=TurnBitOn(0, 0)
                ) as progress:
                    try:
                        with transaction.atomic():
                            progress.setContentsLabel(f'<b>Удаляю: {Attr_type.objects.get(id=id)}, вхождений: {cnt}</b>')
                            for attribute in Document_attributes.objects.filter(attr_type_id=id):
                                qty, _ = Document_attr_cross.objects.select_related('attribute').filter(attribute=attribute).delete()
                                res += qty

                                qty, _ = Lotsman_document_attr_cross.objects.select_related('attribute').filter(attribute=attribute).delete()
                                res += qty

                                attribute.delete()
                                step_res = progress.step()

                                if step_res != 0:
                                    raise ProgressDroped(progress_deleted)

                            qty, _ = super().filter(id=id).delete()
                    except ProgressDroped as ex:
                        Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                        raise ex
            res += qty
        return res

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "code": record.code,
            "name": record.name,
            "full_name": record.full_name,
            "description": record.description,
            "parent_id": record.parent_id,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    @staticmethod
    def get_or_create_attr(attr_codes, attr_names=None, logger=None):
        from django.conf import settings
        settings.LOCKS.acquire('AttrManager.get_or_create_attr')

        attr_codes = attr_codes.split('.')
        attr_names = attr_names.split('.') if attr_names != None else []
        attr_type = None
        parent_attr_type = None
        i = 0
        for attr_code in attr_codes:
            attr_name = None
            try:
                attr_name = attr_names[i]
            except:
                attr_name = attr_codes[i]

            attr_type, create = Attr_type.objects.get_or_create(
                code=attr_code,
                defaults=dict(
                    name=attr_name,
                    parent=parent_attr_type,
                ))
            i += 1

            # if create and logger:
            #     logger.logging(f'Added Parent Attr_type: {attr_type}', 'debug')

            parent_attr_type = attr_type

        if attr_type == None:
            settings.LOCKS.release('AttrManager.get_or_create_attr')
            raise Exception(f'Failed to create type with code {attr_codes}.')

        settings.LOCKS.release('AttrManager.get_or_create_attr')
        return attr_type

    def get_attr(self, code, name=None, default=None):
        try:
            if name:
                return super().get_queryset().get(code=code, name=name)
            else:
                return super().get_queryset().get(code=code)
        except Attr_type.DoesNotExist as e:
            if default:
                return default
            else:
                raise e

    def get_queryset(self):
        return AttrQuerySet(self.model, using=self._db)


class Attr_type(BaseRefHierarcy):
    code = CodeField(verbose_name="Код", db_index=True, unique=True)
    name = NameField(verbose_name="Наименование", db_index=True)
    props = BitField(flags=(
        ('relevant', 'Актуальность'),
    ), default=1, db_index=True)

    relevent_label = "Действует"
    not_relevent_label = "Не действует"

    objects = AttrManager()

    def __str__(self):
        return f"ID={self.id}, code={self.code} name={self.name} props={self.props}"

    class Meta:
        verbose_name = 'Тип атрибута'



# Generated by Django 2.2.8 on 2019-12-16 10:16

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0115_lotsman_documents_hierarcy_props'),
        ('ckk', '0178_auto_20191216_0758'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='lotsman_document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Lotsman_documents_hierarcy', verbose_name='Документ из Лоцмана'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-20 17:57

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0035_auto_20190420_1208'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('from_cdw', 'Получено из чертежа'), ('for_line', 'Строка спецификации')), default=1),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import lotsman_documents_hierarcy_files

urlpatterns = [

    path('Lotsman_documents_hierarcy_files/Fetch/', lotsman_documents_hierarcy_files.Lotsman_documents_hierarcy_files_Fetch),
    path('Lotsman_documents_hierarcy_files/Add', lotsman_documents_hierarcy_files.Lotsman_documents_hierarcy_files_Add),
    path('Lotsman_documents_hierarcy_files/Update', lotsman_documents_hierarcy_files.Lotsman_documents_hierarcy_files_Update),
    path('Lotsman_documents_hierarcy_files/Remove', lotsman_documents_hierarcy_files.Lotsman_documents_hierarcy_files_Remove),
    path('Lotsman_documents_hierarcy_files/Lookup/', lotsman_documents_hierarcy_files.Lotsman_documents_hierarcy_files_Lookup),
    path('Lotsman_documents_hierarcy_files/Info/', lotsman_documents_hierarcy_files.Lotsman_documents_hierarcy_files_Info),
    path('Lotsman_documents_hierarcy_files/Copy', lotsman_documents_hierarcy_files.Lotsman_documents_hierarcy_files_Copy),

]



import logging

from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Documents_historyQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Documents_historyManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Documents_historyQuerySet(self.model, using=self._db)


class Documents_history(AuditModel):
    old_document = ForeignKeyCascade(Documents, related_name='old_document')
    new_document = ForeignKeyCascade(Documents, related_name='new_document')

    objects = Documents_historyManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('old_document', 'new_document'),)



# Generated by Django 2.2 on 2019-04-26 10:17

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0041_status'),
    ]

    operations = [
        migrations.AlterField(
            model_name='status',
            name='code',
            field=isc_common.fields.code_field.CodeField(unique=True),
        ),
    ]
